from starlette.applications import Starlette
from strawberry.contrib.starlette.app.graphql_app import GraphQLApp


def get_graphql_app(schema, tracing=False):
    app = Starlette(debug=True)
    app.add_route("/graphql", GraphQLApp(schema, tracing=tracing))
    return app


def create_query(query, variables=None, operation_name=None):
    return {"query": query, "variables": variables, "operationName": operation_name}



import asyncio

import strawberry
from starlette.testclient import TestClient
from strawberry.contrib.starlette.tests.helpers import create_query, get_graphql_app


def test_starlette_tracing():
    @strawberry.type
    class Query:
        hello: str = "strawberry"

    schema = strawberry.Schema(query=Query)

    app = get_graphql_app(schema, tracing=True)
    client = TestClient(app)

    query = create_query(
        """
            query {
                hello
            }
        """
    )

    response = client.post("/graphql", json=query)
    print(response.content)
    assert response.status_code == 200

    json = response.json()
    assert type(json["extensions"]["tracing"]) == dict


def test_starlette_tracing_async():
    @strawberry.type
    class Query:
        @strawberry.field
        async def hello(self, info) -> str:
            await asyncio.sleep(1)
            return "world"

    schema = strawberry.Schema(query=Query)

    app = get_graphql_app(schema, tracing=True)
    client = TestClient(app)

    query = create_query(
        """
            query {
                hello
            }
        """
    )

    response = client.post("/graphql", json=query)
    print(response.content)
    assert response.status_code == 200

    json = response.json()
    assert type(json["extensions"]["tracing"]) == dict

    tracing_results = json["extensions"]["tracing"]
    assert tracing_results["duration"] >= 1000000000
    assert tracing_results["duration"] <= 1500000000


def test_starlette_tracing_disabled_without_flag():
    @strawberry.type
    class Query:
        hello: str = "strawberry"

    schema = strawberry.Schema(query=Query)

    app = get_graphql_app(schema)
    client = TestClient(app)

    query = create_query(
        """
            query {
                hello
            }
        """
    )

    response = client.post("/graphql", json=query)
    assert response.status_code == 200

    json = response.json()
    assert "extensions" not in json



import functools
import typing
from contextlib import contextmanager

from graphql import MiddlewareManager
from strawberry.types import ExecutionContext

from . import Extension


def run_on_all_extensions(func=None, pass_context=False):
    def decorator(func):
        @functools.wraps(func)
        def wrap(self, *args, **kwargs):
            method_name_start = f"on_{func.__name__}_start"
            method_name_end = f"on_{func.__name__}_end"

            extension_kwargs = (
                {"execution_context": self.execution_context} if pass_context else {}
            )

            for extension in self.extensions:
                getattr(extension, method_name_start)(**extension_kwargs)

            try:
                result = func(self, *args, **kwargs)

                return result

            finally:
                for extension in self.extensions:
                    getattr(extension, method_name_end)(**extension_kwargs)

        return wrap

    if func:
        return decorator(func)

    return decorator


class ExtensionsRunner:
    def __init__(
        self,
        execution_context: ExecutionContext,
        extensions: typing.Sequence[Extension] = None,
    ):
        self.execution_context = execution_context
        self.extensions = extensions or []

    @contextmanager
    @run_on_all_extensions(pass_context=True)
    def request(self):
        yield

    @contextmanager
    @run_on_all_extensions
    def validation(self):
        yield

    @contextmanager
    @run_on_all_extensions
    def parsing(self):
        yield

    def get_extensions_results(self) -> typing.Dict[str, typing.Any]:
        data = {}

        for extension in self.extensions:
            data.update(extension.get_results())

        return data

    def as_middleware_manager(self, *additional_middlewares) -> MiddlewareManager:
        middlewares = tuple(self.extensions) + additional_middlewares

        return MiddlewareManager(*middlewares)
